From d1c8d7fc25c407050c828ad6d88bba9f2c303295 Mon Sep 17 00:00:00 2001
From: Aleksander Machniak <alec@alec.pl>
Date: Sun, 28 Sep 2014 13:03:08 -0400
Subject: [PATCH] Support hashing algorithms without mhash() and even hash() extension (#1490076)

---
 plugins/password/drivers/ldap.php |   73 +++++++++++++++++++++++++++++-------
 1 files changed, 58 insertions(+), 15 deletions(-)

diff --git a/plugins/password/drivers/ldap.php b/plugins/password/drivers/ldap.php
index 0c93237..acd9687 100644
--- a/plugins/password/drivers/ldap.php
+++ b/plugins/password/drivers/ldap.php
@@ -220,75 +220,118 @@
         case 'crypt':
             $crypted_password = '{CRYPT}' . crypt($password_clear, self::random_salt(2));
             break;
+
         case 'ext_des':
             /* Extended DES crypt. see OpenBSD crypt man page */
             if (!defined('CRYPT_EXT_DES') || CRYPT_EXT_DES == 0) {
                 /* Your system crypt library does not support extended DES encryption */
                 return false;
             }
+
             $crypted_password = '{CRYPT}' . crypt($password_clear, '_' . self::random_salt(8));
             break;
+
         case 'md5crypt':
             if (!defined('CRYPT_MD5') || CRYPT_MD5 == 0) {
                 /* Your system crypt library does not support md5crypt encryption */
                 return false;
             }
+
             $crypted_password = '{CRYPT}' . crypt($password_clear, '$1$' . self::random_salt(9));
             break;
+
         case 'blowfish':
             if (!defined('CRYPT_BLOWFISH') || CRYPT_BLOWFISH == 0) {
                 /* Your system crypt library does not support blowfish encryption */
                 return false;
             }
+
             /* Hardcoded to second blowfish version and set number of rounds */
             $crypted_password = '{CRYPT}' . crypt($password_clear, '$2a$12$' . self::random_salt(13));
             break;
+
         case 'md5':
             $crypted_password = '{MD5}' . base64_encode(pack('H*', md5($password_clear)));
             break;
+
         case 'sha':
             if (function_exists('sha1')) {
                 /* Use PHP 4.3.0+ sha1 function, if it is available */
                 $crypted_password = '{SHA}' . base64_encode(pack('H*', sha1($password_clear)));
-            } else if (function_exists('mhash')) {
+            }
+            else if (function_exists('hash')) {
+                $crypted_password = '{SHA}' . base64_encode(hash('sha1', $password_clear, true));
+            }
+            else if (function_exists('mhash')) {
                 $crypted_password = '{SHA}' . base64_encode(mhash(MHASH_SHA1, $password_clear));
-            } else {
-                /* Your PHP install does not have the mhash() function */
+            }
+            else {
+                /* Your PHP install does not have the mhash()/hash() nor sha1() function */
                 return false;
             }
             break;
+
         case 'ssha':
+            mt_srand((double) microtime() * 1000000);
+            $salt = substr(pack('h*', md5(mt_rand())), 0, 8);
+
             if (function_exists('mhash') && function_exists('mhash_keygen_s2k')) {
-                mt_srand((double) microtime() * 1000000 );
-                $salt = mhash_keygen_s2k(MHASH_SHA1, $password_clear, substr(pack('h*', md5(mt_rand())), 0, 8), 4);
-                $crypted_password = '{SSHA}' . base64_encode(mhash(MHASH_SHA1, $password_clear . $salt) . $salt);
-            } else {
-                /* Your PHP install does not have the mhash() function */
+                $salt     = mhash_keygen_s2k(MHASH_SHA1, $password_clear, $salt, 4);
+                $password = mhash(MHASH_MD5, $password_clear . $salt);
+            }
+            else if (function_exists('sha1')) {
+                $salt     = substr(pack("H*", sha1($salt . $password_clear)), 0, 4);
+                $password = sha1($password_clear . $salt, true);
+            }
+            else if (function_exists('hash')) {
+                $salt     = substr(pack("H*", hash('sha1', $salt . $password_clear)), 0, 4);
+                $password = hash('sha1', $password_clear . $salt, true);
+            }
+
+            if ($password) {
+                $crypted_password = '{SSHA}' . base64_encode($password . $salt);
+            }
+            else {
+                /* Your PHP install does not have the mhash()/hash() nor sha1() function */
                 return false;
             }
             break;
+
         case 'smd5':
+            mt_srand((double) microtime() * 1000000);
+            $salt = substr(pack('h*', md5(mt_rand())), 0, 8);
+
             if (function_exists('mhash') && function_exists('mhash_keygen_s2k')) {
-                mt_srand((double) microtime() * 1000000 );
-                $salt = mhash_keygen_s2k(MHASH_MD5, $password_clear, substr(pack('h*', md5(mt_rand())), 0, 8), 4);
-                $crypted_password = '{SMD5}' . base64_encode(mhash(MHASH_MD5, $password_clear . $salt) . $salt);
-            } else {
-                /* Your PHP install does not have the mhash() function */
-                return false;
+                $salt     = mhash_keygen_s2k(MHASH_MD5, $password_clear, $salt, 4);
+                $password = mhash(MHASH_MD5, $password_clear . $salt);
             }
+            else if (function_exists('hash')) {
+                $salt     = substr(pack("H*", hash('md5', $salt . $password_clear)), 0, 4);
+                $password = hash('md5', $password_clear . $salt, true);
+            }
+            else {
+                $salt     = substr(pack("H*", md5($salt . $password_clear)), 0, 4);
+                $password = md5($password_clear . $salt, true);
+            }
+
+            $crypted_password = '{SMD5}' . base64_encode($password . $salt);
             break;
+
         case 'samba':
             if (function_exists('hash')) {
                 $crypted_password = hash('md4', rcube_charset::convert($password_clear, RCUBE_CHARSET, 'UTF-16LE'));
                 $crypted_password = strtoupper($crypted_password);
-            } else {
+            }
+            else {
                 /* Your PHP install does not have the hash() function */
                 return false;
             }
             break;
+
         case 'ad':
             $crypted_password = rcube_charset::convert('"' . $password_clear . '"', RCUBE_CHARSET, 'UTF-16LE');
             break;
+
         case 'clear':
         default:
             $crypted_password = $password_clear;

--
Gitblit v1.9.1